<?php

/*
 * Boilerplate
 */

declare(strict_types=1);

namespace Univie\Basisklassifikation;

if (!defined('\\ABSPATH')) {
    exit();
}


/*
 * Model for a Basisklassifikation class.
 */

class Klasse
{
    protected \stdClass $translations;

    public static function from_json(string $json): Klasse
    {
        return self::from_jskos(
            json_decode(
                $json,
                flags:  JSON_INVALID_UTF8_SUBSTITUTE
                        | JSON_OBJECT_AS_ARRAY
                        | JSON_THROW_ON_ERROR
            )
        );
    }

    public static function from_jskos(array $jskos): Klasse
    {
        assert($jskos['ancestors'][0] === $jskos['broader'][0]);
        assert(count($jskos['broader']) === 0);

        $ancestors = [];
        if (isset($jskos['ancestors'])) {
            foreach (array_reverse($jskos['ancestors']) as $record) {
                $ancestor = self::from_jskos($record);
                $ancestor->ancestors = $ancestors;
                array_unshift($ancestors, $ancestor);
            }
        }

        $class = new self(
            $jskos['notation'][0],
            $jskos['prefLabel'],
            $ancestors,
        );

        if (isset($jskos['narrower'])) {
            array_unshift($ancestors, $class);
            foreach ($jskos['narrower'] as $record) {
                $class->narrower[] = new self(
                    $record['notation'][0],
                    $record['prefLabel'],
                    $ancestors
                );
            }
        }

        return $class;
    }

    public function __construct(
        public string $notation,
        public array $labels,
        public array $ancestors = [],
        public array $narrower = []
    ) {
        $this->translations = new \stdClass();

        foreach (LANGUAGES as $lang) {
            $short = substr($lang, 0, 2);
            if (isset($labels[$short])) {
                $title = sanitize_title($notation);
                $name = "{$notation} {$labels[$short]}";
                $slug = "{$title}-{$short}";

                $this->translations->$lang = compact('name', 'slug');
            }
        }
    }

    public function get_term(string $lang): \WP_Term
    {
        if (!isset($this->translations->$lang)) {
            throw new Errors\NoTranslation($this->notation, $lang);
        }

        $slug = $this->translations->$lang['slug'];

        $term = get_term_by('slug', $slug, TAXONOMY);
        if ($term === false) {
            throw new Errors\NoTerm($slug);
        }

        return $term;
    }

    public function insert_terms(): void
    {
        $this->insert_or_update_terms('insert_term', insert: false);
    }

    public function update_terms(bool $insert = false): void
    {
        $this->insert_or_update_terms('update_term', insert: $insert);
    }

    protected function insert_or_update_terms(
        string $callback,
        bool $insert
    ): void
    {
        $ancestors = $this->ancestors;
        foreach ($ancestors as $ancestor) {
            $ancestor->insert_or_update_terms('update_term', $insert);
        }

        $translations = [];
        foreach (get_object_vars($this->translations) as $lang => $args) {
            $args['parent'] = 0;
            if (count($ancestors) > 0) {
                $args['parent'] = $ancestors[0]->get_term($lang)->term_id;
            }

            try {
                $result = $this->$callback($lang, $args);
            } catch (Errors\NoTerm $e) {
                if (!$insert) {
                    throw $e;
                }

                $result = $this->insert_term($lang, $args);
            } catch (Errors\NoTranslation) {
                continue;
            }

            if (is_wp_error($result)) {
                throw new Errors\WP($result);
            }

            /* TODO: Add notation as metadata field. */

            if (!TRANSLATE) {
                return;
            }

            pll_set_term_language($result['term_id'], $lang);
            $translations[$lang] = $result['term_id'];
        }

        pll_save_term_translations($translations);
    }

    protected function insert_term(string $lang, array $args): array|\WP_Error
    {
        return wp_insert_term($args['name'], TAXONOMY, $args);
    }

    protected function update_term(string $lang, array $args): array|\WP_Error
    {
        return wp_update_term($this->get_term($lang)->term_id, TAXONOMY, $args);
    }
}
